# Default values for options
$repetitions = 1
$minSleep = 1
$maxSleep = 5
$baseUrl = "https://ardzsd.de"
$client = "default_client"
$user = "default_user"
$network = "default_network"
$outputFile = "curl_report.csv"
$cssUrl = "otrs-web/skins/Agent/default/css-cache/CommonCSS_336575c40014056de7baefb417c78be8.css"
$pngUrl = "otrs-web/skins/Agent/BR/img/ARD-ServiceDesk_RGB_72dpi.png"

# Function to perform curl and capture timing data
function Perform-Curl {
    param (
        [string]$url,
        [string]$fileType,
        [int]$iteration
    )
    
    # Generate a random query parameter to avoid cache
    $randomQuery = "nocache=" + [guid]::NewGuid().ToString()
    $fullUrl = "$url?$randomQuery"

    # Get the current date and time for the test
    $timestamp = Get-Date -Format "yyyy-MM-dd HH:mm:ss"
    
    # Perform the curl request, measure time and bypass cache
    $curlOutput = curl.exe -o NUL -s -w "`%{time_starttransfer};`%{time_total}" -H "Cache-Control: no-cache" $fullUrl
    
    $outputParts = $curlOutput -split ';'
    $ttfb = $outputParts[0]
    $totalTime = $outputParts[1]
    $downloadTime = [math]::Round(($totalTime - $ttfb), 3)
    
    # Append results to the CSV file
    Add-Content $outputFile "$timestamp;$url;$ttfb;$downloadTime;$totalTime;$client;$user;$network"
    
    Write-Host "$fileType - Iteration $iteration: TTFB: $ttfb, DownloadTime: $downloadTime; Total Time: $totalTime"
}

# Function to get a random sleep duration between MIN and MAX
function Get-RandomSleep {
    $sleepTime = Get-Random -Minimum $minSleep -Maximum $maxSleep
    Write-Host "Sleeping for $sleepTime seconds..."
    Start-Sleep -Seconds $sleepTime
}

# Parse command-line arguments for repetitions, sleep settings, and other options
param (
    [int]$r = $repetitions,
    [int]$m = $minSleep,
    [int]$x = $maxSleep,
    [string]$u = $baseUrl,
    [string]$c = $client,
    [string]$U = $user,
    [string]$n = $network
)

# Set the CSS and PNG URLs based on the base URL
$cssUrl = "$u/$cssUrl"
$pngUrl = "$u/$pngUrl"

# Initialize the CSV report with headers
Add-Content $outputFile "timestamp;url;ttfb;download;total;client;user;network"

# Loop through the number of repetitions
for ($i = 1; $i -le $r; $i++) {
    Write-Host "Test iteration $i of $r"
    
    # Perform the curl for the CSS file
    Perform-Curl $cssUrl "CSS" $i
    
    # Perform the curl for the PNG file
    Perform-Curl $pngUrl "PNG" $i
    
    # Sleep for a random duration between the tests
    if ($i -lt $r) {
        Get-RandomSleep
    }
}

Write-Host "Test completed. Results saved in $outputFile"